// nporshe.cc --- The Porshe class with native behavior.

// Author:        Didier Verna <didier@lrde.epita.fr>
// Maintainer:    Didier Verna <didier@lrde.epita.fr>
// Created:       Wed Oct 15 10:05:47 2008
// Last Revision: Mon Apr  6 16:11:14 2009


// Commentary:

// Contents management by FCM version 0.1.

// In this version, the Porshe hierarchy is equipped with native behavior
// (paint and recycle) which requires modifying the whole hierarchy by hand.


// Code:

#include <iostream>
#include <ostream>
#include <vector>

// A wheel
struct Wheel
{
  Wheel () { std::cout << "Created a wheel." << std::endl; };

  void paint ()
  {
    std::cout << "Painting a wheel." << std::endl;
  };
  void recycle ()
  {
    std::cout << "Recycling a wheel." << std::endl;
  };
};

// A body with 4 wheels
struct Body
{
  Body ()
  {
    for (int i=0; i<4; i++)
      _wheels.push_back (Wheel());
    std::cout << "Created a body." << std::endl;
  };

  void paint ()
  {
    std::cout << "Painting the body." << std::endl;

    for (std::vector<Wheel>::iterator i = _wheels.begin ();
	 i != _wheels.end ();
	 i++)
      (*i).paint ();
  };
  void recycle ()
  {
    std::cout << "Recycling the body." << std::endl;

    for (std::vector<Wheel>::iterator i = _wheels.begin ();
	 i != _wheels.end ();
	 i++)
      (*i).recycle ();
  };

  std::vector<Wheel> _wheels;
};

// An engine
struct Engine
{
  Engine () { std::cout << "Created an engine." << std::endl; };

  void paint ()
  {
    std::cout << "Painting the engine." << std::endl;
  };
  void recycle ()
  {
    std::cout << "Recycling the engine." << std::endl;
  };
};

// A Porshe with a 4 wheels body and an engine
struct Porshe
{
  Porshe () { std::cout << "Created a Porshe." << std::endl; };

  void paint ()
  {
    std::cout << "Painting the porshe." << std::endl;
    _engine.paint ();
    _body.paint ();
  };
  void recycle ()
  {
    std::cout << "Recycling the porshe." << std::endl;
    _engine.recycle ();
    _body.recycle ();
  };

  Body _body;
  Engine _engine;
};


int main (int argc, char *argv[])
{
  Porshe porshe;

  porshe.paint ();
  porshe.recycle ();
}

// nporshe.cc ends here
