;;; nporshe.lisp --- The Porshe class with native behavior.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Mon Apr  6 17:19:19 2009
;; Last Revision: Mon Apr  6 17:53:33 2009


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:


;; The Porshe hierarchy is untouched because in CLOS, methods do not belong to
;; classes but rather behave as ordinary functions.
(defclass wheel ()
  ())

(defclass body ()
  ((wheels :initform (loop :for i :from 1 :upto 4
			   :for wheel = (make-instance 'wheel)
			   :collect wheel)
	   :accessor wheels)))

(defclass engine ()
  ())

(defclass porshe ()
  ((engine :initform (make-instance 'engine)
	   :accessor engine)
   (body :initform (make-instance 'body)
	 :accessor body)))


;; The native painting behavior
(defgeneric paint (object)
  (:method ((porshe porshe))
    (format t "Painting the Porshe.~%")
    (paint (engine porshe))
    (paint (body porshe)))
  (:method ((engine engine))
    (format t "Painting the engine.~%"))
  (:method ((body body))
    (format t "Painting the body.~%")
    (dolist (wheel (wheels body))
      (paint wheel)))
  (:method ((wheel wheel))
    (format t "Painting a wheel.~%")))

;; The native recycling behavior
(defgeneric recycle (object)
  (:method ((porshe porshe))
    (format t "Recycling the Porsh.~%")
    (recycle (engine porshe))
    (recycle (body porshe)))
  (:method ((engine engine))
    (format t "Recycling the engine.~%"))
  (:method ((body body))
    (format t "Recycling the body.~%")
    (dolist (wheel (wheels body))
      (recycle wheel)))
  (:method ((wheel wheel))
    (format t "Recycling a wheel.~%")))


(defun main ()
  (let ((porshe (make-instance 'porshe)))
    (paint porshe)
    (recycle porshe))
  (quit))


;;; nporshe.lisp ends here
