;;; vporshe.lisp --- The Porshe class with visiting behavior ala C++.

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>
;; Created:       Mon Apr  6 17:50:01 2009
;; Last Revision: Mon Apr  6 18:04:58 2009


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:


;; Nothing in there (generic functions are outside the classes).
(defclass porshe-visitor ()
  ())

;; Same here.
(defclass visitable-porshe-component ()
  ())

(defclass wheel (visitable-porshe-component)
  ())

(defclass body (visitable-porshe-component)
  ((wheels :initform (loop :for i :from 1 :upto 4
			   :for wheel = (make-instance 'wheel)
			   :collect wheel)
	   :accessor wheels)))

(defclass engine (visitable-porshe-component)
  ())

(defclass porshe (visitable-porshe-component)
  ((engine :initform (make-instance 'engine)
	   :accessor engine)
   (body :initform (make-instance 'body)
	 :accessor body)))


;; Let's define the visit generic function for our visitors.
(defgeneric visit (object visitor))

;; Let's define the accept methods, only outside the classes.
(defgeneric accept (object visitor)
  (:method ((wheel wheel) (visitor porshe-visitor))
    (visit wheel visitor))
  (:method ((body body) (visitor porshe-visitor))
    (visit body visitor)
    (dolist (wheel (wheels body))
      (accept wheel visitor)))
  (:method ((engine engine) (visitor porshe-visitor))
    (visit engine visitor))
  (:method ((porshe porshe) (visitor porshe-visitor))
    (visit porshe visitor)
    (accept (engine porshe) visitor)
    (accept (body porshe) visitor)))


;; A painting visitor. Nothing in there.
(defclass paint-porshe-visitor (porshe-visitor)
  ())
(defmethod visit ((wheel wheel) (visitor paint-porshe-visitor))
  (format t "Painting a wheel.~%"))
(defmethod visit ((body body) (visitor paint-porshe-visitor))
  (format t "Painting the body.~%"))
(defmethod visit ((engine engine) (visitor paint-porshe-visitor))
  (format t "Painting the engine.~%"))
(defmethod visit ((porshe porshe) (visitor paint-porshe-visitor))
  (format t "Painting the Porshe.~%"))

;; A recycling visitor. Same here.
(defclass recycle-porshe-visitor (porshe-visitor)
  ())
(defmethod visit ((wheel wheel) (visitor recycle-porshe-visitor))
  (format t "Recycling a wheel.~%"))
(defmethod visit ((body body) (visitor recycle-porshe-visitor))
  (format t "Recycling the body.~%"))
(defmethod visit ((engine engine) (visitor recycle-porshe-visitor))
  (format t "Recycling the engine.~%"))
(defmethod visit ((porshe porshe) (visitor recycle-porshe-visitor))
  (format t "Recycling the Porshe.~%"))


(defun main ()
  (let ((porshe (make-instance 'porshe)))
    (accept porshe (make-instance 'paint-porshe-visitor))
    (accept porshe (make-instance 'recycle-porshe-visitor)))
  (quit))


;;; vporshe.lisp ends here
